@@ -472,21 +472,62 @@ def compare_gfile_to_ods(gfile, ods, require_shot=True, require_efitid=False, re
     return not (device_mismatch + shot_mismatch + efitid_mismatch + times_mismatch)
 
 
+def check_eqdsk_type(xeqdsk, desired_type=None, allow_contains=True):
+    """
+    Checks that an object is an *EQDSK file (g/a/m/k) or a container that contains EQDSKs
+
+    :param xeqdsk: object
+        Object to test
+
+    :param desired_type: type
+        OMFITgeqdsk, OMFITaeqdsk, OMFITmeqdsk, or OMFITkeqdsk.
+        Supply None to accept any of these types.
+
+    :param allow_contains: bool
+        True: The object is valid if it's container that contains the right kind of file.
+        False: The object iself must be the right kind of file.
+
+    :return: bool
+        xeqdsk is valid according to the desired type and container rules
+    """
+
+    if desired_type is None:
+        desired_type = [OMFITgeqdsk, OMFITmeqdsk, OMFITaeqdsk, OMFITkeqdsk]
+
+    x_valid = isinstance(xeqdsk, desired_type)
+    if allow_contains and isinstance(xeqdsk, dict):
+        xfiles = [k for k, v in xeqdsk.items() if isinstance(v, desired_type)]
+        x_valid = x_valid or bool(len(xfiles))
+    return x_valid
+
+
 def efitviewer_tree_selector(show_status_label=True):
     """
     Setup for a data source in the OMFIT tree
 
     :param show_status_label: bool
     """
+    from omfit_classes.omfit_omas_utils import check_shot_time
+
     data_source = 'tree'
-    try:
-        gtime0 = root['OUTPUTS']['gEQDSK'].keys()[0]
-        default_g_file = "root['OUTPUTS']['gEQDSK']['{}']".format(gtime0)
-    except (KeyError, IndexError):
-        default_g_file = 'unused'
-    OMFITx.TreeLocationPicker(
-        settings_loc + "['cases'][{}]['gfile']".format(index), 'G-file(s) in tree', default=default_g_file, updateGUI=True
-    )
+    for gamk in 'gamk':
+        gamk_io = 'INPUTS' if gamk == 'k' else 'OUTPUTS'
+        try:
+            gamk_time0 = root[gamk_io]['gEQDSK'].keys()[0]
+            default_gamk_file = f"root['{gamk_io}']['{gamk}EQDSK']['{gamk_time0}']"
+        except (KeyError, IndexError):
+            default_gamk_file = 'unused'
+        OMFITx.TreeLocationPicker(
+            f"{settings_loc}['cases'][{index}]['{gamk}file']",
+            f'{gamk.upper()}-file(s) in tree',
+            default=default_gamk_file,
+            updateGUI=True,
+            help='G-files provide the essential data for plotting the equilibrium. A-files contain additional '
+            'quantities. M-files contain measurements that can be used as inputs as well as reconstructed values '
+            'to compare to those measaurements and chi-squared values. K-files contain constraints and setup '
+            'instructions.',
+        )
+
     is_g = contains_g = None
     try:
         gfile_str = settings['cases'][index]['gfile']
@@ -506,10 +547,60 @@ def efitviewer_tree_selector(show_status_label=True):
     else:
         if is_g:
             case_info = ' '.join(gfile['CASE'])
+            shot = int(gfile['CASE'][3].split('#')[-1])
+            t = float(gfile['CASE'][4].split('ms')[0])
         elif contains_g:
             case_info = ' '.join(array(gfile[gfiles[0]]['CASE'])[array([0, 1, 2, 3, 5])])
+            shot = int(gfile[gfiles[0]]['CASE'][3].split('#')[-1])
+            t = np.array([float(gf['CASE'][4].split('ms')[0]) for gf in gfile.values()])
         else:
             case_info = ''
+            shot = None
+            t = None
+    if gfile_valid:
+        try:
+            afile = eval(settings['cases'][index]['afile'])
+        except (KeyError, IndexError, TypeError, NameError):
+            afile = None
+            afile_valid = False
+        else:
+            afile_valid = check_shot_time(shot, t, afile, raise_on_bad=False) and check_eqdsk_type(afile, OMFITaeqdsk)
+
+        try:
+            mfile = eval(settings['cases'][index]['mfile'])
+        except (KeyError, IndexError, TypeError, NameError):
+            mfile = None
+            mfile_valid = False
+        else:
+            mfile_valid = check_shot_time(shot, t, mfile, raise_on_bad=False) and check_eqdsk_type(mfile, OMFITmeqdsk)
+
+        try:
+            kfile = eval(settings['cases'][index]['kfile'])
+        except (KeyError, IndexError, TypeError, NameError):
+            kfile = None
+            kfile_valid = False
+        else:
+            kfile_valid = check_shot_time(shot, t, kfile, raise_on_bad=False) and check_eqdsk_type(kfile, OMFITkeqdsk)
+
+        not_included = []
+
+        if afile_valid:
+            case_info += '\n+aEQDSK data'
+        else:
+            not_included += ['aEQDSK']
+        if mfile_valid:
+            case_info += '\n+mEQDSK data'
+        else:
+            not_included += ['mEQDSK']
+        if kfile_valid:
+            case_info += '\n+kEQDSK data'
+        else:
+            not_included += ['kEQDSK']
+        if len(not_included):
+            not_included = 'Not included: ' + ', '.join(not_included)
+    else:
+        afile_valid = mfile_valid = kfile_valid = None
+        not_included = []
 
     ods_loaded = ods_tag in out
 
@@ -533,11 +624,13 @@ def efitviewer_tree_selector(show_status_label=True):
         select_text_color = 'black'
     else:
         select_text_var.set('Select an OMFITgeqdsk or container of OMFITgeqdsk instances to continue.')
-        select_text_color = 'black'
+        select_text_color = 'red'
 
     add_status_label(index, data_source, show_status_label, color=select_text_color)
     if gfile_valid:
         OMFITx.Button('{} {}'.format(re_load, case_info), lambda: load_cx_ods(index=index), updateGUI=True)
+        if len(not_included):
+            OMFITx.Label(not_included, fg='red')
     # else:
     # OMFITx.Label('Select an OMFITgeqdsk or container of OMFITgeqdsk instances to continue')
     return ods_tag
