@@ -277,7 +277,10 @@ class OMFITcoils(SortedDict, OMFITascii):
             coils[key]['i_flag'] = i_flag
             coils[key]['length'] = length
             coils[key]['l_flag'] = l_flag
-            coils[key]['target_length'] = target_length
+            if target_length < 0:
+                coils[key]['target_length'] = length
+            else:
+                coils[key]['target_length'] = target_length
             coils[key]['nfcoil'] = nfcoil
             coils[key]['xc'] = csx[0::2]
             coils[key]['xs'] = csx[1::2]
