@@ -149,7 +149,18 @@ def add_command_box_tab():
 
 # Data loading and related helpers --------------------------------------------------------------------------------
 def parse_load_cx_ods_kw(
-    gfile=None, device=None, shot=None, t=None, efitid=None, index=0, no_empty=None, settings=None, minimal_eq_data=None
+    gfile=None,
+    afile=None,
+    mfile=None,
+    kfile=None,
+    device=None,
+    shot=None,
+    t=None,
+    efitid=None,
+    index=0,
+    no_empty=None,
+    settings=None,
+    minimal_eq_data=None,
 ):
     """
     Handles setting defaults for keywords to load_cx_ods()
@@ -162,33 +173,62 @@ def parse_load_cx_ods_kw(
 
     :return: tuple containing
         gfile: OMFITgeqdsk or None
+        afile: OMFITaeqdsk or None
+        mfile: OMFITmeqdsk or None
+        kfile: OMFITkeqdsk or None
         device: str
         shot: int
         efitid: str
         no_empty: bool
+        minimal_eq_data: bool
         ods_tag: str
     """
     if settings is None:
         _, _, _, settings, _, _ = pick_output_location()
     the_case = settings['cases'][index]
-    def_def_gfile = settings['cases'][0]['gfile']
-    def_def_device = tokamak(settings['cases'][0]['device'])
-    def_def_shot = evalExpr(settings['cases'][0]['shot'])
+    def_def_gfile = settings['cases'][0].setdefault('gfile', None)
+    def_def_afile = settings['cases'][0].setdefault('afile', None)
+    def_def_mfile = settings['cases'][0].setdefault('mfile', None)
+    def_def_kfile = settings['cases'][0].setdefault('kfile', None)
+    def_def_device = tokamak(settings['cases'][0].setdefault('device', None))
+    def_def_shot = evalExpr(settings['cases'][0].setdefault('shot', 0))
     if index == 0:
         ods_tag = 'efitviewer_ods'
         def_device = def_def_device
         def_shot = def_def_shot
         def_gfile = def_def_gfile
+        def_afile = def_def_afile
+        def_mfile = def_def_mfile
+        def_kfile = def_def_kfile
     else:
         ods_tag = 'efitviewer_ods_{}'.format(index)
         def_device = the_case.setdefault('device', def_def_device)
         def_shot = the_case.setdefault('shot', def_def_shot)
         def_gfile = the_case.setdefault('gfile', def_def_gfile)
+        def_afile = the_case.setdefault('afile', def_def_afile)
+        def_mfile = the_case.setdefault('mfile', def_def_mfile)
+        def_kfile = the_case.setdefault('kfile', def_def_kfile)
 
     if gfile is None:
         gfile = def_gfile
     if gfile == 'unused':
         gfile = None
+
+    if afile is None:
+        afile = def_afile
+    if afile == 'unused':
+        afile = None
+
+    if mfile is None:
+        mfile = def_mfile
+    if mfile == 'unused':
+        mfile = None
+
+    if kfile is None:
+        kfile = def_kfile
+    if kfile == 'unused':
+        kfile = None
+
     if device is None:
         device = def_device
     if shot is None:
@@ -200,14 +240,61 @@ def parse_load_cx_ods_kw(
     if minimal_eq_data is None:
         minimal_eq_data = settings.setdefault('minimal_eq_data', True)
 
-    return gfile, device, shot, efitid, no_empty, minimal_eq_data, ods_tag
+    return gfile, afile, mfile, kfile, device, shot, efitid, no_empty, minimal_eq_data, ods_tag
 
 
-def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0, no_empty=None, minimal_eq_data=None):
+def read_gamk_file_strings(device, gfile, afile, mfile, kfile):
+    """
+    Processes inputs that could be strings containing locations instead of the actual target objects
+    :param device: str
+    :param gfile: str or OMFITgeqdsk
+    :param afile: str or OMFITaeqdsk
+    :param mfile: str or OMFITmeqdsk
+    :param kfile: str or OMFITkeqdsk
+    :return: tuple
+        gfile: OMFITgeqdsk or None
+        gfile_str: str or None
+        afile: OMFITaeqdsk or None
+        afile_str: str or None
+        mfile: OMFITmeqdsk or None
+        mfile_str: str or None
+        kfile: OMFITkeqdsk or None
+        kfile_str: str or None
+    """
+    result = []
+    for x in 'gamk':
+        varname = f'{x}file'
+        var = eval(varname)
+        if isinstance(var, str):
+            var_str = var
+            var = eval(var)
+        elif isinstance_str(var, f'OMFIT{x}eqdsk'):
+            var_str = f'<<YOUR {x.upper}-FILE FOR {device}:{var.filename}>>'
+        elif isinstance(var, dict):
+            var_str = f'<<A group of {x}EQDSK files for {device}>>'
+        else:
+            var_str = None
+        result += [var, var_str]
+    return result
+
+
+def load_cx_ods(
+    gfile=None,
+    afile=None,
+    mfile=None,
+    kfile=None,
+    device=None,
+    shot=None,
+    t=None,
+    efitid=None,
+    index=0,
+    no_empty=None,
+    minimal_eq_data=None,
+):
     """
     Puts an ODS with CX info in the tree
 
-    :param gfile: OMFITgeqdsk instance [optional if shot, t, and efitid are supplied]
+    :param gfile,afile,mfile,kfile: OMFIT[g/a/m/k]eqdsk instance [optional if shot, t, and efitid are supplied]
         Pass in 'unused' to prevent None from being used as an instruction to seek out a default value.
 
     :param device: string
@@ -234,8 +321,11 @@ def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0
     """
     printd(f'  loading cx ods for index {index}', topic='omfit_efitviewer')
     _, _, out, settings, _, _ = pick_output_location()
-    gfile, device, shot, efitid, no_empty, minimal_eq_data, ods_tag = parse_load_cx_ods_kw(
+    gfile, afile, mfile, kfile, device, shot, efitid, no_empty, minimal_eq_data, ods_tag = parse_load_cx_ods_kw(
         gfile=gfile,
+        afile=afile,
+        mfile=mfile,
+        kfile=kfile,
         device=device,
         shot=shot,
         t=t,
@@ -246,13 +336,7 @@ def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0
         minimal_eq_data=minimal_eq_data,
     )
 
-    if isinstance(gfile, str):
-        gfile_str = gfile
-        gfile = eval(gfile)
-    elif gfile is not None:
-        gfile_str = '<<YOUR G-FILE FOR {device:}:{gfilename:}>>'.format(device=device, gfilename=gfile.filename)
-    else:
-        gfile_str = None
+    gfile, gfile_str, afile, afile_str, mfile, mfile_str, kfile, kfile_str = read_gamk_file_strings(device, gfile, afile, mfile, kfile)
 
     if gfile is None:
         load_cmd = "setup_hardware_overlay_cx({}, {}, {}, efitid={}, default_load=False, no_empty={}, minimal_eq_data={})".format(
@@ -265,7 +349,15 @@ def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0
     else:
         printd(f' Using a gfile or set with type {type(gfile)} to setup cx data.', topic='omfit_efitviewer')
         ods = out[ods_tag] = setup_hardware_overlay_cx(
-            device, geqdsk=gfile, default_load=False, quiet=True, no_empty=no_empty, minimal_eq_data=minimal_eq_data
+            device,
+            geqdsk=gfile,
+            aeqdsk=afile,
+            meqdsk=mfile,
+            keqdsk=kfile,
+            default_load=False,
+            quiet=True,
+            no_empty=no_empty,
+            minimal_eq_data=minimal_eq_data,
         )
         load_cmd = "setup_hardware_overlay_cx({}, geqdsk={}, default_load=False, no_empty={}, minimal_eq_data={})".format(
             repr(device), gfile_str, no_empty, minimal_eq_data
