@@ -796,7 +796,7 @@ class OMFITmhdin(OMFITnamelist):
 
         return self
 
-    def to_omas(self, ods=None, update=['pf_active', 'magnetics']):
+    def to_omas(self, ods=None, update=['oh', 'pf_active', 'flux_loop', 'b_field_pol_probe']):
         """
         Transfers data in EFIT mhdin.dat format to ODS
 
@@ -806,8 +806,10 @@ class OMFITmhdin(OMFITnamelist):
         :param ods: ODS instance
             Data will be added in-place
 
-        :param update: IDS to populate
-            ['pf_active', 'magnetics']
+        :param update: systems to populate
+            ['oh', 'pf_active', 'flux_loop', 'b_field_pol_probe']
+            ['magnetics'] will enable both ['flux_loop', 'b_field_pol_probe']
+            NOTE that in IMAS the OH information goes under `pf_active`
 
         :return: ODS instance
         """
@@ -818,6 +820,27 @@ class OMFITmhdin(OMFITnamelist):
         if ods is None:
             ods = ODS()
 
+        # oh
+        if 'oh' in update:
+            coil_data = self['OH']
+            turns = self['IN3']['ECTURN']
+            elements_id = (self['IN3']['ECID'] - 1).astype(int)
+            rect_code = geo_type_lookup('rectangle', 'pf_active', ods.imas_version, reverse=True)
+            for i in range(len(coil_data)):
+                c = elements_id[i]
+                e = sum(elements_id[:i] == elements_id[i])
+                ods['pf_active.coil'][c]['name'] = f'OH{c}'
+                ods['pf_active.coil'][c]['identifier'] = f'OH{c}'
+                ods['pf_active.coil'][c]['element'][e]['name'] = f'OH{c}_{e}'
+                ods['pf_active.coil'][c]['element'][e]['identifier'] = f'OH{c}_{e}'
+                ods['pf_active.coil'][c]['element'][e]['turns_with_sign'] = turns[i]
+                rect = ods['pf_active.coil'][c]['element'][e]['geometry.rectangle']
+                rect['r'] = coil_data[i, 0]
+                rect['z'] = coil_data[i, 1]
+                rect['width'] = coil_data[i, 2]
+                rect['height'] = coil_data[i, 3]
+                ods['pf_active.coil'][c]['element'][e]['geometry.geometry_type'] = rect_code
+
         # pf_active
         if 'pf_active' in update:
             coil_data = self['FC']
@@ -825,9 +848,10 @@ class OMFITmhdin(OMFITnamelist):
             elements_id = (self['IN3']['FCID'] - 1).astype(int)
             rect_code = geo_type_lookup('rectangle', 'pf_active', ods.imas_version, reverse=True)
             outline_code = geo_type_lookup('outline', 'pf_active', ods.imas_version, reverse=True)
+            offset = len(ods['pf_active.coil'])
             for i in range(len(coil_data)):
-                c = elements_id[i]
-                e = sum(elements_id[:i] == c)
+                c = elements_id[i] + offset
+                e = sum(elements_id[:i] == elements_id[i])
                 ods['pf_active.coil'][c]['name'] = f'PF{c}'
                 ods['pf_active.coil'][c]['identifier'] = f'PF{c}'
                 ods['pf_active.coil'][c]['element'][e]['name'] = f'PF{c}_{e}'
@@ -837,12 +861,12 @@ class OMFITmhdin(OMFITnamelist):
                     rect = ods['pf_active.coil'][c]['element'][e]['geometry.rectangle']
                     rect['r'] = coil_data[i, 0]
                     rect['z'] = coil_data[i, 1]
-                    rect['width'] = coil_data[i, 2]  # Or width in R
-                    rect['height'] = coil_data[i, 3]  # Or height in Z
+                    rect['width'] = coil_data[i, 2]
+                    rect['height'] = coil_data[i, 3]
                     ods['pf_active.coil'][c]['element'][e]['geometry.geometry_type'] = rect_code
                 else:
                     outline = ods['pf_active.coil'][c]['element'][e]['geometry.outline']
-                    fdat = coil_data[i]
+                    fdat = copy.deepcopy(coil_data[i])
                     fdat[4] = -coil_data[i, 4] * np.pi / 180.0
                     fdat[5] = -(coil_data[i, 5] * np.pi / 180.0 if coil_data[i, 5] != 0 else np.pi / 2)
                     outline['r'] = [
@@ -859,16 +883,11 @@ class OMFITmhdin(OMFITnamelist):
                     ]
                     ods['pf_active.coil'][c]['element'][e]['geometry.geometry_type'] = outline_code
 
-        # magnetics
-        if 'magnetics' in update:
+        # flux_loop
+        if 'magnetics' in update or 'flux_loop' in update:
             R_flux_loop = self['IN3']['RSI']
             Z_flux_loop = self['IN3']['ZSI']
             name_flux_loop = list(map(lambda x: x.strip(), self['IN3']['LPNAME']))
-            R_magnetic = self['IN3']['XMP2']
-            Z_magnetic = self['IN3']['YMP2']
-            A_magnetic = self['IN3']['AMP2']
-            S_magnetic = self['IN3']['SMP2']
-            name_magnetic = list(map(lambda x: x.strip(), self['IN3']['MPNAM2']))
             with omas_environment(ods, cocosio=1):
                 for k, (r, z, name) in enumerate(zip(R_flux_loop, Z_flux_loop, name_flux_loop)):
                     ods[f'magnetics.flux_loop.{k}.name'] = name
@@ -877,6 +896,14 @@ class OMFITmhdin(OMFITnamelist):
                     ods[f'magnetics.flux_loop.{k}.position[0].z'] = z
                     ods[f'magnetics.flux_loop.{k}.type.index'] = 1
 
+        # b_field_pol_probe
+        if 'magnetics' in update or 'b_field_pol_probe' in update:
+            R_magnetic = self['IN3']['XMP2']
+            Z_magnetic = self['IN3']['YMP2']
+            A_magnetic = self['IN3']['AMP2']
+            S_magnetic = self['IN3']['SMP2']
+            name_magnetic = list(map(lambda x: x.strip(), self['IN3']['MPNAM2']))
+            with omas_environment(ods, cocosio=1):
                 for k, (r, z, a, s, name) in enumerate(zip(R_magnetic, Z_magnetic, A_magnetic, S_magnetic, name_magnetic)):
                     ods[f'magnetics.b_field_pol_probe.{k}.name'] = name
                     ods[f'magnetics.b_field_pol_probe.{k}.identifier'] = name
