@@ -88,26 +88,37 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     if shot < 100775:
         warnings.warn('DIII-D Gas valve locations not applicable for shots earlier than 100775 (2000 JAN 17)')
 
-    i = 0
+    imas_version = getattr(ods, 'imas_version', None)
+    separate_valves = imas_version is not None and compare_version(imas_version, "3.34.0") >= 0
+
+    i = 0  # Pipe index
+    iv = 0  # Valve index
 
     def pipe_copy(pipe_in):
         pipe_out = ods['gas_injection']['pipe'][i]
         for field in ['name', 'exit_position.r', 'exit_position.z', 'exit_position.phi']:
             pipe_out[field] = copy.copy(pipe_in[field])
-        vvv = 0
-        while 'valve.{}.identifier'.format(vvv) in pipe_in:
-            pipe_out = copy.copy(pipe_in['valve.{}.identifier'.format(vvv)])
-            vvv += 1
-        return pipe_out
+        if separate_valves:
+            valve_out = ods['gas_injection']['valve'][iv]
+            pipe_out['valve_indices'] = [iv]
+        else:
+            valve_out = None
+        return pipe_out, valve_out
 
     # PFX1
+    pipe_indices = []
     for angle in [12, 139, 259]:  # degrees, DIII-D hardware left handed coords
         pipe_pfx1 = ods['gas_injection']['pipe'][i]
         pipe_pfx1['name'] = 'PFX1_{:03d}'.format(angle)
         pipe_pfx1['exit_position']['r'] = 1.286  # m
         pipe_pfx1['exit_position']['z'] = 1.279  # m
         pipe_pfx1['exit_position']['phi'] = -np.pi / 180.0 * angle  # radians, right handed
-        pipe_pfx1['valve'][0]['identifier'] = 'PFX1'
+        if separate_valves:
+            pipe_pfx1['valve_indices'] = [iv]
+        else:
+            pipe_pfx1['valve'][0]['identifier'] = 'PFX1'
+        pipe_indices += [i]
+
         dr = -1.116 + 1.286
         dz = -1.38 + 1.279
         # pipea['exit_position']['direction'] = 180/np.pi * tan(dz/dr) if dr != 0 else 90 * sign(dz)
@@ -115,16 +126,33 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
         pipe_pfx1['second_point']['r'] = pipe_pfx1['exit_position']['r'] + dr
         pipe_pfx1['second_point']['z'] = pipe_pfx1['exit_position']['z'] + dz
         i += 1
+    if separate_valves:
+        valve_pfx1 = ods['gas_injection']['valve'][iv]
+        valve_pfx1['identifier'] = 'PFX1'
+        valve_pfx1['name'] = 'PFX1'
+        valve_pfx1['pipe_indices'] = pipe_indices
+    iv += 1
 
     # PFX2 injects at the same poloidal locations as PFX1, but at different toroidal angles
+    pipe_indices = []
     for angle in [79, 199, 319]:  # degrees, DIII-D hardware left handed coords
         pipe_copy(pipe_pfx1)
         pipe_pfx2 = ods['gas_injection']['pipe'][i]
         pipe_pfx2['name'] = 'PFX2_{:03d}'.format(angle)
         pipe_pfx2['exit_position']['phi'] = -np.pi / 180.0 * angle  # rad
-        pipe_pfx2['valve'][0]['identifier'] = 'PFX2'
+        if separate_valves:
+            pipe_pfx2['valve_indices'] = [iv]
+        else:
+            pipe_pfx2['valve'][0]['identifier'] = 'PFX2'
+        pipe_indices += [i]
         pipe_pfx2['second_point']['phi'] = pipe_pfx2['exit_position']['phi']
         i += 1
+    if separate_valves:
+        valve_pfx2 = ods['gas_injection']['valve'][iv]
+        valve_pfx2['identifier'] = 'PFX2'
+        valve_pfx2['name'] = 'PFX2'
+        valve_pfx2['pipe_indices'] = pipe_indices
+    iv += 1
 
     # GAS A
     pipea = ods['gas_injection']['pipe'][i]
@@ -132,19 +160,31 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipea['exit_position']['r'] = 1.941  # m
     pipea['exit_position']['z'] = 1.01  # m
     pipea['exit_position']['phi'] = -np.pi / 180.0 * 300  # rad
-    pipea['valve'][0]['identifier'] = 'GASA'
     # pipea['exit_position']['direction'] = 270.  # degrees, giving dir of pipe leading towards injector, up is 90
     pipea['second_point']['phi'] = pipea['exit_position']['phi']
     pipea['second_point']['r'] = pipea['exit_position']['r']
     pipea['second_point']['z'] = pipea['exit_position']['z'] - 0.01
+    if separate_valves:
+        pipea['valve_indices'] = [iv]
+        valvea = ods['gas_injection']['valve'][iv]
+        valvea['identifier'] = 'GASA'
+        valvea['name'] = 'GASA'
+        valvea['pipe_indices'] = [i]
+    else:
+        pipea['valve'][0]['identifier'] = 'GASA'
     i += 1
+    iv += 1
 
     # GAS B injects in the same place as GAS A
-    pipe_copy(pipea)
-    pipeb = ods['gas_injection']['pipe'][i]
+    pipeb, valveb = pipe_copy(pipea)
     pipeb['name'] = 'GASB_300'
-    pipeb['valve'][0]['identifier'] = 'GASB'
+    if separate_valves:
+        valveb['name'] = valveb['identifier'] = 'GASB'
+        valveb['pipe_indices'] = [i]
+    else:
+        pipeb['valve'][0]['identifier'] = 'GASB'
     i += 1
+    iv += 1
 
     # GAS C
     pipec = ods['gas_injection']['pipe'][i]
@@ -152,13 +192,21 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipec['exit_position']['r'] = 1.481  # m
     pipec['exit_position']['z'] = -1.33  # m
     pipec['exit_position']['phi'] = -np.pi / 180.0 * 0
-    pipec['valve'][0]['identifier'] = 'GASC'
-    pipec['valve'][1]['identifier'] = 'GASE'
     # pipec['exit_position']['direction'] = 90.  # degrees, giving direction of pipe leading towards injector
     pipec['second_point']['phi'] = pipec['exit_position']['phi']
     pipec['second_point']['r'] = pipec['exit_position']['r']
     pipec['second_point']['z'] = pipec['exit_position']['z'] + 0.01
+    if separate_valves:
+        valvec = ods['gas_injection']['valve'][iv]
+        valvec['name'] = 'GASC'
+        valvec['identifier'] = 'GASC'
+        valvec['pipe_indices'] = [i]
+        pipec['valve_indices'] = [iv]
+    else:
+        pipec['valve'][0]['identifier'] = 'GASC'
+
     i += 1
+    iv += 1
 
     # GAS D injects at the same poloidal location as GAS A, but at a different toroidal angle.
     # There is a GASD piezo valve that splits into four injectors, all of which have their own gate valves and can be
@@ -167,54 +215,99 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     piped = ods['gas_injection']['pipe'][i]
     piped['name'] = 'GASD_225'  # This is the injector name
     piped['exit_position']['phi'] = -np.pi / 180.0 * 225
-    piped['valve'][0]['identifier'] = 'GASD'  # This is the piezo name
     piped['second_point']['phi'] = piped['exit_position']['phi']
+    if separate_valves:
+        valved = ods['gas_injection']['valve'][iv]
+        valved['name'] = valved['identifier'] = 'GASD'  # This is the piezo name
+        valved['pipe_indices'] = [i]
+        piped['valve_indices'] = [iv]
+    else:
+        piped['valve'][0]['identifier'] = 'GASD'
     i += 1
+    iv += 1
 
-    # Spare 225 is an extra branch of the GASD line after the GASD piezo
+    # GASE is at the same poloidal location as GASC
+    pipee, valvee = pipe_copy(pipec)
+    if separate_valves:
+        valvee['name'] = valvee['identifier'] = 'GASE'
+        valvee['pipe_indices'] = [i]
+    else:
+        pipee['valve'][0]['identifier'] = 'GASE'
+    i += 1
+    iv += 1
+
+    # Spare 225 is an extra branch of the GASD line after the GASD piezo (might not have a valve)
     pipe_copy(piped)
     pipes225 = ods['gas_injection']['pipe'][i]
     pipes225['name'] = 'Spare_225'  # This is the injector name
     i += 1
 
     # RF_170 and RF_190: gas ports near the 180 degree antenna, on the GASD line
+    # I don't think these are connected anymore, but I don't know the shot number after which they were removed.
     for angle in [170, 190]:
         pipe_rf = ods['gas_injection']['pipe'][i]
         pipe_rf['name'] = 'RF_{:03d}'.format(angle)
         pipe_rf['exit_position']['r'] = 2.38  # m
         pipe_rf['exit_position']['z'] = -0.13  # m
         pipe_rf['exit_position']['phi'] = -np.pi / 180.0 * angle  # rad
-        pipe_rf['valve'][0]['identifier'] = 'GASD'
         i += 1
 
     # DRDP
     pipe_copy(piped)
-    piped = ods['gas_injection']['pipe'][i]
-    piped['name'] = 'DRDP_225'
-    piped['valve'][0]['identifier'] = 'DRDP'
+    pipedrdp = ods['gas_injection']['pipe'][i]
+    pipedrdp['name'] = 'DRDP_225'
+    if separate_valves:
+        pipedrdp['valve_indices'] = [iv]
+        valvedrdp = ods['gas_injection']['valve'][iv]
+        valvedrdp['identifier'] = 'DRDP'
+        valvedrdp['name'] = 'DRDP'
+        valvedrdp['pipe_indices'] = [i]
+    else:
+        pipedrdp['valve'][0]['identifier'] = 'DRDP'
     i += 1
+    iv += 1
 
     # UOB
+    pipe_indices = []
     for angle in [45, 165, 285]:
         pipe_uob = ods['gas_injection']['pipe'][i]
         pipe_uob['name'] = 'UOB_{:03d}'.format(angle)
         pipe_uob['exit_position']['r'] = 1.517  # m
         pipe_uob['exit_position']['z'] = 1.267  # m
         pipe_uob['exit_position']['phi'] = -np.pi / 180.0 * angle
-        pipe_uob['valve'][0]['identifier'] = 'UOB'
+        pipe_indices += [i]
+        if separate_valves:
+            pipe_uob['valve_indices'] = [iv]
+        else:
+            pipe_uob['valve'][0]['identifier'] = 'UOB'
         # pipe_uob['exit_position']['direction'] = 270.  # degrees, giving dir of pipe leading to injector, up is 90
         i += 1
+    if separate_valves:
+        valve_uob = ods['gas_injection']['valve'][iv]
+        valve_uob['name'] = valve_uob['identifier'] = 'UOB'
+        valve_uob['pipe_indices'] = pipe_indices
+    iv += 1
 
     # LOB1
+    pipe_indices = []
     for angle in [30, 120]:
         pipe_lob1 = ods['gas_injection']['pipe'][i]
         pipe_lob1['name'] = 'LOB1_{:03d}'.format(angle)
         pipe_lob1['exit_position']['r'] = 1.941  # m
         pipe_lob1['exit_position']['z'] = -1.202  # m
         pipe_lob1['exit_position']['phi'] = -np.pi / 180.0 * angle
-        pipe_lob1['valve'][0]['identifier'] = 'LOB1'
+        if separate_valves:
+            pipe_lob1['valve_indices'] = [iv]
+        else:
+            pipe_lob1['valve'][0]['identifier'] = 'LOB1'
+        pipe_indices += [i]
         # pipe_lob1['exit_position']['direction'] = 180.  # degrees, giving dir of pipe leading to injector; up is 90
         i += 1
+    if separate_valves:
+        valve_lob1 = ods['gas_injection']['valve'][iv]
+        valve_lob1['name'] = valve_lob1['identifier'] = 'LOB1'
+        valve_lob1['pipe_indices'] = pipe_indices
+    iv += 1
 
     # Spare 75 is an extra branch of the GASC line after the LOB1 piezo
     pipes75 = ods['gas_injection']['pipe'][i]
@@ -222,7 +315,11 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipes75['exit_position']['r'] = 2.249  # m (approximate / estimated from still image)
     pipes75['exit_position']['z'] = -0.797  # m (approximate / estimated from still image)
     pipes75['exit_position']['phi'] = 75  # degrees, DIII-D hardware left handed coords
-    pipes75['valve'][0]['identifier'] = 'LOB1'
+    if separate_valves:
+        pipes75['valve_indices'] = pipe_lob1['valve_indices']
+        valve_lob1['pipe_indices'] = np.append(valve_lob1['pipe_indices'], [i])
+    else:
+        pipes75['valve'][0]['identifier'] = 'LOB1'
     # pipes75['exit_position']['direction'] = 180.  # degrees, giving direction of pipe leading towards injector
     i += 1
 
@@ -233,7 +330,11 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
         pipe_rf_lob1['exit_position']['r'] = 2.38  # m
         pipe_rf_lob1['exit_position']['z'] = -0.13  # m
         pipe_rf_lob1['exit_position']['phi'] = -np.pi / 180.0 * angle
-        pipe_rf_lob1['valve'][0]['identifier'] = 'LOB1'
+        if separate_valves:
+            pipe_rf_lob1['valve_indices'] = pipe_lob1['valve_indices']
+            valve_lob1['pipe_indices'] = np.append(valve_lob1['pipe_indices'], [i])
+        else:
+            pipe_rf_lob1['valve'][0]['identifier'] = 'LOB1'
         # pipe_rf10['exit_position']['direction'] = 180.  # degrees, giving dir of pipe leading to injector; up is 90
         i += 1
 
@@ -243,10 +344,17 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_dimesc['exit_position']['r'] = 1.481  # m
     pipe_dimesc['exit_position']['z'] = -1.33  # m
     pipe_dimesc['exit_position']['phi'] = -np.pi / 180.0 * 165
-    pipe_dimesc['valve'][0]['identifier'] = '240R-2'
-    pipe_dimesc['valve'][0]['name'] = '240R-2 (PCS use GASD)'
+    if separate_valves:
+        pipe_dimesc['valve_indices'] = [iv]
+        cpgas_valve = ods['gas_injection']['valve'][iv]
+        cpgas_valve['identifier'] = '240R-2'
+        cpgas_valve['name'] = '240R-2 (PCS use GASD)'
+        cpgas_valve['pipe_indices'] = [i]
+    else:
+        pipe_dimesc['valve'][0]['identifier'] = '240R-2'
     # pipe_dimesc['exit_position']['direction'] = 90.  # degrees, giving dir of pipe leading towards injector, up is 90
     i += 1
+    iv += 1
 
     # CPBOT
     pipe_cpbot = ods['gas_injection']['pipe'][i]
@@ -254,26 +362,43 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_cpbot['exit_position']['r'] = 1.11  # m
     pipe_cpbot['exit_position']['z'] = -1.33  # m
     pipe_cpbot['exit_position']['phi'] = -np.pi / 180.0 * 150
-    pipe_cpbot['valve'][0]['identifier'] = '240R-2'
-    pipe_cpbot['valve'][0]['name'] = '240R-2 (PCS use GASD)'
+    if separate_valves:
+        pipe_cpbot['valve_indices'] = pipe_dimesc['valve_indices']
+        cpgas_valve['pipe_indices'] = np.append(cpgas_valve['pipe_indices'], [i])
+    else:
+        pipe_cpbot['valve'][0]['identifier'] = '240R-2'
     # pipe_cpbot['exit_position']['direction'] = 0.  # degrees, giving dir of pipe leading towards injector, up is 90
     i += 1
 
     # LOB2 injects at the same poloidal locations as LOB1, but at different toroidal angles
+    if separate_valves:
+        valve_lob2 = ods['gas_injection']['valve'][iv]
+        valve_lob2['identifier'] = 'LOB2'
+        valve_lob2['name'] = 'LOB2'
+        valve_lob2['pipe_indices'] = []
     for angle in [210, 300]:
         pipe_copy(pipe_lob1)
         pipe_lob2 = ods['gas_injection']['pipe'][i]
         pipe_lob2['name'] = 'LOB2_{:03d}'.format(angle)
         pipe_lob2['exit_position']['phi'] = -np.pi / 180.0 * angle  # degrees, DIII-D hardware left handed coords
-        pipe_lob2['valve'][0]['identifier'] = 'LOB2'
+        if separate_valves:
+            pipe_lob2['valve_indices'] = [iv]
+            valve_lob2['pipe_indices'] = np.append(valve_lob2['pipe_indices'], [i])
+        else:
+            pipe_lob2['valve'][0]['identifier'] = 'LOB2'
         i += 1
+    iv += 1
 
     # Dimes floor tile 165
     pipe_copy(pipec)
     pipe_dimesf = ods['gas_injection']['pipe'][i]
     pipe_dimesf['name'] = 'DiMES_Tile_160'
     pipe_dimesf['exit_position']['phi'] = -np.pi / 180.0 * 165
-    pipe_dimesf['valve'][0]['identifier'] = 'LOB2'
+    if separate_valves:
+        pipe_dimesf['valve_indices'] = pipe_lob2['valve_indices']
+        valve_lob2['pipe_indices'] = np.append(valve_lob2['pipe_indices'], [i])
+    else:
+        pipe_dimesf['valve'][0]['identifier'] = 'LOB2'
     i += 1
 
     # RF COMB
@@ -282,7 +407,11 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_rfcomb['exit_position']['r'] = 2.38  # m
     pipe_rfcomb['exit_position']['z'] = -0.13  # m
     # pipe_rfcomb['exit_position']['phi'] = Unknown, sorry
-    pipe_rfcomb['valve'][0]['identifier'] = 'LOB2'
+    if separate_valves:
+        pipe_rfcomb['valve_indices'] = pipe_lob2['valve_indices']
+        valve_lob2['pipe_indices'] = np.append(valve_lob2['pipe_indices'], [i])
+    else:
+        pipe_rfcomb['valve'][0]['identifier'] = 'LOB2'
     # pipe_rf307['exit_position']['direction'] = 180.  # degrees, giving dir of pipe leading towards injector, up is 90
     i += 1
 
@@ -292,7 +421,11 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_rf307['exit_position']['r'] = 2.38  # m
     pipe_rf307['exit_position']['z'] = -0.13  # m
     pipe_rf307['exit_position']['phi'] = -np.pi / 180.0 * 307
-    pipe_rf307['valve'][0]['identifier'] = 'LOB2'
+    if separate_valves:
+        pipe_rf307['valve_indices'] = pipe_lob2['valve_indices']
+        valve_lob2['pipe_indices'] = np.append(valve_lob2['pipe_indices'], [i])
+    else:
+        pipe_rf307['valve'][0]['identifier'] = 'LOB2'
     # pipe_rf307['exit_position']['direction'] = 180.  # degrees, giving dir of pipe leading towards injector, up is 90
     i += 1
 
@@ -300,21 +433,24 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_copy(pipec)
     pipeh = ods['gas_injection']['pipe'][i]
     pipeh['name'] = 'GASH_000'
-    pipeh['valve'][0]['identifier'] = '???'  # This one's not on the manifold schematic
+    if separate_valves:
+        pipeh['valve_indices'] = []  # This one's not on the manifold schematic
     i += 1
 
     # GAS I injects in the same place as GAS C
     pipe_copy(pipec)
     pipei = ods['gas_injection']['pipe'][i]
     pipei['name'] = 'GASI_000'
-    pipei['valve'][0]['identifier'] = '???'  # This one's not on the manifold schematic
+    if separate_valves:
+        pipei['valve_indices'] = []  # This one's not on the manifold schematic
     i += 1
 
     # GAS J injects in the same place as GAS D
     pipe_copy(piped)
     pipej = ods['gas_injection']['pipe'][i]
     pipej['name'] = 'GASJ_225'
-    pipej['valve'][0]['identifier'] = '???'  # This one's not on the manifold schematic
+    if separate_valves:
+        pipej['valve_indices'] = []  # This one's not on the manifold schematic
     i += 1
 
     # RF260
@@ -323,7 +459,11 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_rf260['exit_position']['r'] = 2.38  # m
     pipe_rf260['exit_position']['z'] = 0.14  # m
     pipe_rf260['exit_position']['phi'] = -np.pi / 180.0 * 260
-    pipe_rf260['valve'][0]['identifier'] = 'LOB2?'  # Seems to have been removed. May have been on LOB2, though.
+    if separate_valves:
+        pipe_rf260['valve_indices'] = pipe_lob2['valve_indices']  # Seems to have been removed. May have been on LOB2, though.
+        valve_lob2['pipe_indices'] = np.append(valve_lob2['pipe_indices'], [i])
+    else:
+        pipe_rf260['valve'][0]['identifier'] = 'LOB2'
     # pipe_rf260['exit_position']['direction'] = 180.  # degrees, giving dir of pipe leading towards injector, up is 90
     i += 1
 
@@ -332,7 +472,8 @@ def setup_gas_injection_hardware_description_d3d(ods, shot):
     pipe_cpmid['name'] = 'CPMID'
     pipe_cpmid['exit_position']['r'] = 0.9  # m
     pipe_cpmid['exit_position']['z'] = -0.2  # m
-    pipe_cpmid['valve'][0]['identifier'] = '???'  # Seems to have been removed. Not on schematic.
+    if separate_valves:
+        pipe_cpmid['valve_indices'] = []  # Seems to have been removed. Not on schematic.
     # pipe_cpmid['exit_position']['direction'] = 0.  # degrees, giving dir of pipe leading towards injector, up is 90
     i += 1
 
