@@ -268,7 +268,7 @@ class OMFITrdb(SortedDict):
     def postgresql(self):
         return self.db == 'd3d' or self.server == 'huez' or self.server == 'pppl_postgres'
 
-    def _connect(self):
+    def _connect(self, redact_password=True):
         """Connects to the database"""
         connection = self._setup_connection()
         try:
@@ -305,9 +305,14 @@ If this file is not available to you contact the GA computer group: iris-support
                 self.cnxn = pyodbc.connect(**connection2)
                 self.cursor = self.cnxn.cursor()
             except Exception as _excp2:
-                printe('\nFailed connection details: ' + repr(_excp1))
+                msg1 = '\nFailed connection details: ' + repr(_excp1)
+                msg2 = '\nFailed connection details: ' + repr(_excp2)
+                if redact_password:
+                    msg1 = msg1.replace(connection['PWD'], '<PASSWORD REDACTED>')
+                    msg2 = msg2.replace(connection['PWD'], '<PASSWORD REDACTED>')
+                printe(msg1)
                 printe(connection)
-                printe('\nFailed connection details: ' + repr(_excp2))
+                printe(msg2)
                 printe(connection2)
                 printe('\n')
                 raise
