@@ -172,7 +172,7 @@ class OMFITobject(object):
         self.link = None  # used by the OMFITojbect class to copy itself
 
         # stop here if nothing else needs to be done
-        if filename == None:
+        if filename is None:
             return
 
         # generate a temporary directory name (safe for parallel runs)
@@ -253,7 +253,7 @@ class OMFITobject(object):
                 _allOMFITobjects.setdefault(self.filename, []).append(weakref.ref(self))
                 return
 
-        # this is the full thrue path fo the filename
+        # this is the full true path fo the filename
         filename = os.path.expandvars(os.path.expanduser(str(filename)))
 
         # handle comma separated filenames
@@ -271,7 +271,7 @@ class OMFITobject(object):
         if modifyOriginal or OMFITaux['noCopyToCWD'] or noCopyToCWD:
             if not os.path.exists(filename):
                 raise IOError("No such file or directory: " + repr(filename))
-            filename = os.path.abspath(filename)
+            filename = os.path.realpath(filename)
             self.filename = filename
             self.originalFilename = filename
             self.link = filename
