@@ -1,6 +1,6 @@
 from omfit_classes.startup_framework import *
 from matplotlib import _pylab_helpers
-from matplotlib import rcParams
+from matplotlib import rcParams, cm
 from omfit_classes.omfit_dmp import OMFITdmp
 from omfit_classes.omfit_data import Dataset, DataArray
 from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
@@ -36,12 +36,17 @@ def _patch(obj, fun):
 class Figure(matplotlib.figure.Figure):
     def __init__(self, *args, **kw):
         matplotlib.figure.Figure.__init__(self, *args, **kw)
+        self.cbar_cids = []
 
     def colorbar(self, mappable, cax=None, ax=None, use_gridspec=True, **kw):
         """
         Customized default grid_spec=True for consistency with tight_layout.\n\n
         """
-        return matplotlib.figure.Figure.colorbar(self, mappable, cax=cax, ax=ax, use_gridspec=use_gridspec, **kw)
+        cbar = matplotlib.figure.Figure.colorbar(self, mappable, cax=cax, ax=ax, use_gridspec=use_gridspec, **kw)
+        drg_cbar = DraggableColorbar(cbar, mappable)
+        # save weak reference
+        self.cbar_cids.append([drg_cbar.connect(), drg_cbar])
+        return cbar
 
     def printlines(self, filename, squeeze=False):
         """
@@ -317,6 +322,82 @@ class savedFigure(object):
         return OMFITfigure(pickle.loads(self.figurePickle)).figure
 
 
+class DraggableColorbar(object):
+    def __init__(self, cbar, mappable):
+        self.cbar = cbar
+
+        self.mappable = mappable
+        self.press = None
+        self.cycle = sorted([i for i in dir(cm) if hasattr(getattr(cm, i), 'N')])
+        self.index = self.cycle.index(cbar.cmap.name)
+
+    def connect(self):
+        """connect to all the events we need"""
+        self.cidpress = self.cbar.patch.figure.canvas.mpl_connect('button_press_event', self.on_press)
+        self.cidrelease = self.cbar.patch.figure.canvas.mpl_connect('button_release_event', self.on_release)
+        self.cidmotion = self.cbar.patch.figure.canvas.mpl_connect('motion_notify_event', self.on_motion)
+
+    def on_press(self, event):
+        """on button press we will see if the mouse is over us and store some data"""
+        if event.inaxes != self.cbar.ax:
+            return
+        self.press = event.ydata
+
+    def key_press(self, event):
+        if event.key == 'down':
+            self.index += 1
+        elif event.key == 'up':
+            self.index -= 1
+        if self.index < 0:
+            self.index = len(self.cycle)
+        elif self.index >= len(self.cycle):
+            self.index = 0
+        cmap = self.cycle[self.index]
+        self.cbar.cmap = cmap
+        self.cbar.draw_all()
+        self.mappable.set_cmap(cmap)
+        self.cbar.get_axes().set_title(cmap)
+        self.cbar.patch.figure.canvas.draw()
+
+    def on_motion(self, event):
+        'on motion we will move the rect if the mouse is over us'
+        if self.press is None:
+            return
+        if event.inaxes != self.cbar.ax:
+            return
+
+        yprev = self.press
+        ylim = self.cbar.ax.get_ylim()
+        dy = (event.ydata - yprev) / (ylim[1] - ylim[0])
+        self.press = event.ydata
+
+        scale = self.cbar.norm.vmax - self.cbar.norm.vmin
+        perc = 0.03
+        cmin, cmax = self.cbar.ax.get_ylim()
+
+        if event.button == 1:
+            self.cbar.norm.vmax -= scale * dy
+
+        if event.button == 3:
+            self.cbar.norm.vmin -= scale * dy
+
+        self.cbar.draw_all()
+        self.mappable.set_norm(self.cbar.norm)
+        self.cbar.patch.figure.canvas.draw()
+
+    def on_release(self, event):
+        """on release we reset the press data"""
+        self.press = None
+        self.mappable.set_norm(self.cbar.norm)
+        self.cbar.patch.figure.canvas.draw()
+
+    def disconnect(self):
+        """disconnect all the stored connection ids"""
+        self.cbar.patch.figure.canvas.mpl_disconnect(self.cidpress)
+        self.cbar.patch.figure.canvas.mpl_disconnect(self.cidrelease)
+        self.cbar.patch.figure.canvas.mpl_disconnect(self.cidmotion)
+
+
 # enriched figure
 class OMFITfigure(object):  # SortedDict):
     def __init__(self, obj, figureButtons=True):
@@ -1668,7 +1749,6 @@ class FigureNotebook(object):
             fig_kwargs.setdefault('num', label)
             return pyplot.figure(**fig_kwargs)
 
-
         if label in self.figures:
             return self.figures[label]
         fig_kwargs['figsize'] = self.figsize
